package notepad;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

import lazyutil.gui.LzCallback;
import lazyutil.gui.LzDialog;
import lazyutil.gui.LzFrame;
import lazyutil.gui.LzLookAndFeel;
import lazyutil.gui.LzMenu;

/**
 * Sample of LazyGUI
 * @author Hiroyuki Shiota
 */
public class JavaNotepad {

	private LzLookAndFeel laf = LzLookAndFeel.getInstance();

	private LzFrame frame;

	private JScrollPane scrollPanel;

	private JTextArea textArea;

	private JLabel statusLabel;

	private File editFile;

	private String lastPath;

	private LzDialog searchDialog;

	private LzDialog replaceDialog;

	private String lastText = "";

	private static String lineSeparator = System.getProperty("line.separator");

	private JPopupMenu popup;
	
	private DropTarget dropTarget;

	private final CutAction cutAction = new CutAction();

	private final CopyAction copyAction = new CopyAction();

	private final PasteAction pasteAction = new PasteAction();

	private final DeleteAction deleteAction = new DeleteAction();

	private final CutAction cutAction2 = new CutAction();

	private final CopyAction copyAction2 = new CopyAction();

	private final PasteAction pasteAction2 = new PasteAction();

	private final DeleteAction deleteAction2 = new DeleteAction();

	private final UndoAction undoAction = new UndoAction();

	private final RedoAction redoAction = new RedoAction();

	/**
	 * RXgN^
	 *
	 */
	public void LazyNotepad() {
	}

	/**
	 * m[gpbhA\B
	 * @param argFile
	 */
	public void initNotepad(String argFile) {
		laf.toSystem();
		laf.setFont(new Font("Monospaced", Font.PLAIN, 12));
		frame = new LzFrame() {
			/**
			 * E~{^ł̏ItbN
			 */
			protected void processWindowEvent(WindowEvent e) {
				if (!(e.getID() == WindowEvent.WINDOW_CLOSING))
					return;
				if (changedText(new ActionEvent(frame, 0, this.toString()))) {
					super.processWindowEvent(e);
				}
			}
		};
		frame.setTitle("() - JavaNotepad");
		textArea = new JTextArea();
		textArea.setWrapStyleWord(true);
		textArea.setTabSize(4);
		textArea.getDocument().addUndoableEditListener(undoHandler);
		//textArea.setBorder(BorderFactory.createEtchedBorder());
		scrollPanel = new JScrollPane(textArea);

		//ENbNj[ǉB
		popup = new JPopupMenu();
		popup.add(undoAction);
		//popup.add(redoAction);
		popup.addSeparator();
		popup.add(cutAction2);
		popup.add(copyAction2);
		popup.add(pasteAction2);
		popup.add(deleteAction2);
		textArea.setComponentPopupMenu(popup);
		
		//NXŃhbv̓w
		dropTarget = new DropTarget(textArea, new DropTargetAdapter(){
		    public void drop( DropTargetDropEvent e ) {
		        try {
		            Transferable transfer = e.getTransferable();
		            if ( transfer.isDataFlavorSupported( DataFlavor.javaFileListFlavor ) ) {
		                e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
		                java.util.List fileList = (java.util.List)( transfer.getTransferData( DataFlavor.javaFileListFlavor ) ); 
		                readFile((File)fileList.get(0));
		        		textArea.getCaret().setDot(0);
		            }
		        } catch (Exception ex) {
		            //ex.printStackTrace();
		        }
		    }
		});

		frame.getContentPane().add(scrollPanel, BorderLayout.CENTER);
		statusLabel = new JLabel("Welcome");
		frame.getContentPane().add(statusLabel, BorderLayout.SOUTH);
		LzMenu menu = new LzMenu();
		menu.addMenu("t@C(F)/VK(N),Ctrl+N", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				changedText(e);
			}

		});
		menu.addMenu("t@C(F)/J(O),Ctrl+O", new LzCallback(this, "fileOpen"));
		menu.addMenu("t@C(F)/㏑ۑ(S),Ctrl+S",
				new LzCallback(this, "fileSave"));
		menu.addMenu("t@C(F)/Oĕۑ(A)", new LzCallback(this, "fileSaveAs"));
		menu.addMenu("t@C(F)/-");
		menu.addMenu("t@C(F)/̏I(X)", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				if (changedText(e)) {
					frame.dispose();
				}
			}
		});
		menu.addMenu("ҏW(E)/ɖ߂(U),Ctrl+Z", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				try {
					undoAction.actionPerformed(e);
				} catch (Throwable t) {
				}
			}
		});
		menu.addMenu("ҏW(E)/蒼(R),Ctrl+Y", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				try {
					redoAction.actionPerformed(e);
				} catch (Throwable t) {
				}
			}
		});
		menu.addMenu("ҏW(E)/-");
		menu.addMenu("ҏW(E)/؂(T),Ctrl+X", cutAction);
		menu.addMenu("ҏW(E)/Rs[(C),Ctrl+C", copyAction);
		menu.addMenu("ҏW(E)/\t(P),Ctrl+V", pasteAction);
		menu.addMenu("ҏW(E)/폜(L)", deleteAction);
		menu.addMenu("ҏW(E)/-");
		menu.addMenu("ҏW(E)/(F),Ctrl+F", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				replaceDialog.setVisible(false);//ւ
				searchDialog.setVisible(true);
				searchDialog.toFront();
			}
		});
		menu.addMenu("ҏW(E)/u(R),Ctrl+R", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				searchDialog.setVisible(false);//ւ
				replaceDialog.setVisible(true);
				replaceDialog.toFront();
			}
		});
		menu.addMenu("ҏW(E)/-");
		menu.addMenu("ҏW(E)/SđI(A),Ctrl+A", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				textArea.selectAll();
			}
		});
		menu.addMenu("X^C(S)/(O)/E[Ő܂Ԃ(W)", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				scrollPanel
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
				textArea.setLineWrap(true);
			}
		});
		menu.addMenu("X^C(S)/(O)/E[Ő܂ԂȂ(X)", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				scrollPanel
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
				textArea.setLineWrap(false);
			}
		});
		LzCallback fontSizeCallback = new LzCallback(this, "styleFontSize");
		menu.addMenu("X^C(S)/tHgTCY(F)/9", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/10", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/11", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/12", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/13", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/14", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/15", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/16", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/17", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/18", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/19", fontSizeCallback);
		menu.addMenu("X^C(S)/tHgTCY(F)/20", fontSizeCallback);
		//menu.addMenu("Callback/Test", new LCallback(this, "callbackTest"));
		menu.addMenu("O(L)/VXe", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				try {
					laf.toSystem();
					updateLaf();
				} catch (Exception ex) {
					System.err.println("Error loading L&F: " + ex);
				}
			}

		});
		menu.addMenu("O(L)/NXvbgtH[", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				try {
					laf.toCrossPlatform();
					updateLaf();
				} catch (Exception ex) {
					System.err.println("Error loading L&F: " + ex);
				}
			}
		});
		menu.addMenu("O(L)/-");
		menu.addMenu("O(L)/Windows", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				try {
					laf.toWindows();
					updateLaf();
				} catch (Exception ex) {
					System.err.println("Error loading L&F: " + ex);
				}
			}
		});
		menu.addMenu("O(L)/Metal", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				try {
					laf.toMetal();
					updateLaf();
				} catch (Exception ex) {
					System.err.println("Error loading L&F: " + ex);
				}
			}
		});
		menu.addMenu("O(L)/Motif", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				try {
					laf.toMotif();
					updateLaf();
				} catch (Exception ex) {
					System.err.println("Error loading L&F: " + ex);
				}
			}
		});
		menu.addMenu("wv(H)/o[W(A)", new LzCallback(this, "helpVersion"));
		frame.setJMenuBar(menu.getMenuBar());
		if (argFile != null) readFile(new File(argFile));
		frame.setVisible(true);
		frame.toFront();
	}

	private void initSearchDialog() {
		searchDialog = new LzDialog("", 487,129);
		searchDialog.setAlwaysOnTop(true);
		searchDialog.setEscapeToDispose(true);
		searchDialog.setLocation(100,100);
		searchDialog.getContentPane().setLayout(new BorderLayout());
		SearchDialogLeftPanel leftPanel = new SearchDialogLeftPanel();
		leftPanel.init();
		SearchDialogRightPanel rightPanel = new SearchDialogRightPanel();
		rightPanel.init();
		searchDialog.getContentPane().add(leftPanel, BorderLayout.CENTER);
		searchDialog.getContentPane().add(rightPanel, BorderLayout.EAST);
		//searchDialog.setVisible(true);
		//searchDialog.toFront();
	}

	private void initReplaceDialog() {
		replaceDialog = new LzDialog("u", 487,177);
		replaceDialog.setAlwaysOnTop(true);
		replaceDialog.setEscapeToDispose(true);
		replaceDialog.setLocation(100,100);
		replaceDialog.getContentPane().setLayout(new BorderLayout());
		ReplaceDialogLeftPanel leftPanel = new ReplaceDialogLeftPanel();
		leftPanel.init();
		ReplaceDialogRightPanel rightPanel = new ReplaceDialogRightPanel();
		rightPanel.init();
		replaceDialog.getContentPane().add(leftPanel, BorderLayout.CENTER);
		replaceDialog.getContentPane().add(rightPanel, BorderLayout.EAST);
		//replaceDialog.setVisible(true);
		//replaceDialog.toFront();
	}

	/**
	 * p{^
	 */
	private JTextField searchTarget;
	//private JRadioButton downRadioButton;

	private JTextField replaceTarget;
	private JTextField replaceValue;

	private final class CutAction extends AbstractAction {
		public CutAction() {
			super("؂");
		}
		public void actionPerformed(ActionEvent e) {
			textArea.cut();
		}
	}

	private final class CopyAction extends AbstractAction {
		public CopyAction() {
			super("Rs[");
		}
		public void actionPerformed(ActionEvent e) {
			textArea.copy();
		}
	}

	private final class PasteAction extends AbstractAction {
		public PasteAction() {
			super("\t");
		}
		public void actionPerformed(ActionEvent e) {
			textArea.paste();
		}
	}

	private final class DeleteAction extends AbstractAction {
		public DeleteAction() {
			super("폜");
		}
		public void actionPerformed(ActionEvent e) {
			textArea.replaceSelection("");
		}
	}

	private final class SearchDialogLeftPanel extends JPanel {
		private JPanel northPanel = new JPanel();
		private JPanel southPanel = new JPanel();
		//private ButtonGroup buttonGroup = new ButtonGroup();
		//private JRadioButton upRadioButon = new JRadioButton("");
		void init() {
			searchTarget = new JTextField(40);
			this.setLayout(new BorderLayout());
			this.add(northPanel, BorderLayout.CENTER);
			this.add(southPanel, BorderLayout.SOUTH);
			northPanel.setLayout(new FlowLayout());
			northPanel.add(new JLabel("镶 "));
			northPanel.add(searchTarget);
			southPanel.setLayout(new FlowLayout());
			//southPanel.add(new JCheckBox("啶Əʂ"));
			//downRadioButton = new JRadioButton("");
			//buttonGroup.add(upRadioButon);
			//buttonGroup.add(downRadioButton);
			//southPanel.add(upRadioButon);
			//southPanel.add(downRadioButton);
			//downRadioButton.setSelected(true);
		}
	}

	private final class SearchDialogRightPanel extends JPanel {
		public Dimension getPreferredSize() {
			return new Dimension(100,100);
		}
		Action next = new AbstractAction(""){
			public void actionPerformed(ActionEvent e) {
				int start = indexOfSearchText(textArea.getText(), textArea.getCaret().getDot());
				if (start == -1) {
					JOptionPane.showMessageDialog(searchDialog,
							"\"" + searchTarget.getText() + "\" ܂B", "JavaNotepad",
							JOptionPane.INFORMATION_MESSAGE);
					return;
				}
				textArea.setSelectionStart(start);
				int end = start+searchTarget.getText().length();
				textArea.setSelectionEnd(end);
			}
		};
		JButton nextButton = new JButton(next) {
			public Dimension getPreferredSize() {
				return new Dimension(100,24);
			}
		};
		Action cancel = new AbstractAction("LZ"){
			public void actionPerformed(ActionEvent e) {
				searchDialog.setVisible(false);
			}
		};
		JButton cancelButton = new JButton(cancel){
			public Dimension getPreferredSize() {
				return new Dimension(100,24);
			}
		};
		void init(){
			this.setLayout(new FlowLayout());
			this.add(nextButton);
			this.add(cancelButton);
			searchTarget.addKeyListener(new KeyAdapter(){
				public void keyPressed(KeyEvent e) {
					super.keyPressed(e);
					if (e.getKeyCode() == KeyEvent.VK_ENTER) {
						next.actionPerformed(null);
					} else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
						searchDialog.setVisible(false);
					}
				}
			});
		}
	}

	private int indexOfSearchText(String value, int dot) {
		return value.indexOf(searchTarget.getText(), dot);
	}

	private int indexOfReplaceText(String value, int dot) {
		return value.indexOf(replaceTarget.getText(), dot);
	}

	private final class ReplaceDialogLeftPanel extends JPanel {
		private JPanel northPanel = new JPanel();
		private JPanel southPanel = new JPanel();
		//private ButtonGroup buttonGroup = new ButtonGroup();
		//private JRadioButton upRadioButon = new JRadioButton("");
		//private JRadioButton downRadioButton = new JRadioButton("");
		void init() {
			replaceTarget = new JTextField(40);
			replaceValue = new JTextField(40);
			this.setLayout(new BorderLayout());
			this.add(northPanel, BorderLayout.CENTER);
			this.add(southPanel, BorderLayout.SOUTH);
			northPanel.setLayout(new FlowLayout());
			northPanel.add(new JLabel("镶 "));
			northPanel.add(replaceTarget);
			northPanel.add(new JLabel("u̕ "));
			northPanel.add(replaceValue);
			southPanel.setLayout(new FlowLayout());
			//southPanel.add(new JCheckBox("啶Əʂ"));
			//buttonGroup.add(upRadioButon);
			//buttonGroup.add(downRadioButton);
			//southPanel.add(upRadioButon);
			//southPanel.add(downRadioButton);
			//downRadioButton.setSelected(true);
		}
	}

	private final class ReplaceDialogRightPanel extends JPanel {
		private boolean isReplaceEnd = false;
		private boolean isReplaceAll = false;
		public Dimension getPreferredSize() {
			return new Dimension(100,100);
		}
		Action next = new AbstractAction(""){
			public void actionPerformed(ActionEvent e) {
				int start = -1;
				if (isAllSelect()) {
					textArea.getCaret().setDot(0);
					start = indexOfReplaceText(textArea.getText(), 0);
				} else {
					start = indexOfReplaceText(textArea.getText(), textArea.getCaret().getDot());
				}
				if (start == -1) {
					if (! isReplaceAll) {
						JOptionPane.showMessageDialog(replaceDialog,
								"\"" + replaceTarget.getText() + "\" ܂B", "JavaNotepad",
								JOptionPane.INFORMATION_MESSAGE);
					}
					return;
				}
				textArea.setSelectionStart(start);
				int end = start+replaceTarget.getText().length();
				textArea.setSelectionEnd(end);
			}
			public boolean isAllSelect() {
				//System.out.println("Mark:" + textArea.getCaret().getMark() + " Dot:" + textArea.getCaret().getDot() + " Len:" + textArea.getText().length());
				if (
						(textArea.getCaret().getMark() == 0)
						&&
						(textArea.getCaret().getDot() == textArea.getText().length())
				) {
					return true;
				} else {
					return false;
				}
			}
		};
		JButton nextButton = new JButton(next) {
			public Dimension getPreferredSize() {
				return new Dimension(100,24);
			}
		};
		Action replace = new AbstractAction("uĎ"){
			public void actionPerformed(ActionEvent e) {
				int markDot = textArea.getCaret().getMark();
				if (textArea.getCaret().getDot() < markDot) markDot = textArea.getCaret().getDot();
				int start = indexOfReplaceText(textArea.getText(), markDot);
				if (start == -1) {
					if (! isReplaceAll) {
						JOptionPane.showMessageDialog(replaceDialog,
								"\"" + replaceTarget.getText() + "\" ܂B", "JavaNotepad",
								JOptionPane.INFORMATION_MESSAGE);
					}
					isReplaceEnd = true;
					return;
				}
				int end = start+replaceTarget.getText().length();
				textArea.replaceRange(replaceValue.getText(), ((start-1) < 0) ? 0 : start, end);
				//int end2 = start+replaceValue.getText().length();
				next.actionPerformed(e);
			}
		};
		JButton replaceButton = new JButton(replace) {
			public Dimension getPreferredSize() {
				return new Dimension(100,24);
			}
		};
		Action replaceAll = new AbstractAction("SĒu"){
			public void actionPerformed(ActionEvent e) {
				textArea.getCaret().setDot(0);
				isReplaceAll = true;
				isReplaceEnd = false;
				while (! isReplaceEnd) {
					replace.actionPerformed(e);
				}
				isReplaceAll = false;
				isReplaceEnd = false;
			}
		};
		JButton replaceAllButton = new JButton(replaceAll) {
			public Dimension getPreferredSize() {
				return new Dimension(100,24);
			}
		};
		Action cancel = new AbstractAction("LZ"){
			public void actionPerformed(ActionEvent e) {
				replaceDialog.setVisible(false);
			}
		};
		JButton cancelButton = new JButton(cancel){
			public Dimension getPreferredSize() {
				return new Dimension(100,24);
			}
		};
		void init(){
			this.setLayout(new FlowLayout());
			this.add(nextButton);
			this.add(replaceButton);
			this.add(replaceAllButton);
			this.add(cancelButton);
			replaceTarget.addKeyListener(new KeyAdapter(){
				public void keyPressed(KeyEvent e) {
					super.keyPressed(e);
					if (e.getKeyCode() == KeyEvent.VK_ENTER) {
						next.actionPerformed(null);
					} else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
						replaceDialog.setVisible(false);
					}
				}
			});
			replaceValue.addKeyListener(new KeyAdapter(){
				public void keyPressed(KeyEvent e) {
					super.keyPressed(e);
					if (e.getKeyCode() == KeyEvent.VK_ENTER) {
						next.actionPerformed(null);
					} else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
						replaceDialog.setVisible(false);
					}
				}
			});
		}
	}

	/**
	 * t@CJB
	 * @param e
	 */
	public void fileOpen(ActionEvent e) {
		JFileChooser open = new JFileChooser(lastPath);
		open.setFileSelectionMode(JFileChooser.FILES_ONLY);
		open.setMultiSelectionEnabled(false);
		open.showOpenDialog(frame);
		File file = open.getSelectedFile();
		lastPath = file.getParent();
		readFile(file);
		textArea.getCaret().setDot(0);
	}

	/**
	 * t@CǂށB
	 * @param file
	 */
	private void readFile(File file) {
		if (file == null || !file.exists())
			return;
		String line = null;
		StringBuffer sb = new StringBuffer();
		BufferedReader in = null;
		try {
			in = new BufferedReader(new FileReader(file));
			while ((line = in.readLine()) != null) {
				sb.append(line);
				sb.append("\n"/*System.getenv("line.separator")*/);
			}
			textArea.setText(new String(sb));
			statusLabel.setText(file.getAbsolutePath() + " : read ok.");
			editFile = file;
			frame.setTitle(file.getName() + " - JavaNotepad");
			lastText = new String(sb);
			undo = new UndoManager();
			undoHandler = new UndoHandler();
			textArea.getDocument().addUndoableEditListener(undoHandler);
		} catch (Exception ex) {
			ex.printStackTrace();
			statusLabel.setText(file.getAbsolutePath() + " : read error.");
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (Exception ex2) {
				}
			}
		}
	}

	/**
	 * t@CۑB
	 * @param e
	 */
	public void fileSave(ActionEvent e) {
		if (editFile == null) {
			fileSaveAs(e);
		} else {
			writeFile(editFile);
			lastText = textArea.getText();
		}
	}

	public void fileSaveAs(ActionEvent e) {
		JFileChooser save = new JFileChooser(lastPath);
		save.setFileSelectionMode(JFileChooser.FILES_ONLY);
		save.setMultiSelectionEnabled(false);
		save.showSaveDialog(frame);
		File file = save.getSelectedFile();
		if (file == null)
			return;
		if (file.exists()) {
			int result = JOptionPane.showConfirmDialog(frame, file
					.getAbsolutePath()
					+ "͊ɑ݂܂B\n㏑܂?", "Otĕۑ",
					JOptionPane.YES_NO_OPTION);
			if (result != JOptionPane.YES_OPTION)
				return;
		}
		writeFile(file);
		editFile = file;
		frame.setTitle(file.getName() + " - JavaNotepad");
		lastText = textArea.getText();
	}

	/**
	 * eLXgɕύXꍇ̏B
	 * @param e
	 */
	private boolean changedText(ActionEvent e) {
		if (!lastText.equals(textArea.getText())) {
			String fileName = null;
			if (editFile == null) {
				fileName = "";
			} else {
				fileName = editFile.getName();
			}
			int retCode = JOptionPane.showConfirmDialog(frame, "t@C "
					+ fileName + " ̓e͕ύXĂ܂B\n\nύXۑ܂?");
			if (retCode == JOptionPane.CANCEL_OPTION
					|| retCode == JOptionPane.CLOSED_OPTION) {
				return false;
			} else if (retCode == JOptionPane.YES_OPTION) {
				fileSaveAs(e);
			} else if (retCode == JOptionPane.NO_OPTION) {
				editFile = null;
				frame.setTitle("(Untitled) - JavaNotepad");
				textArea.setText("");
			} else {
				return false;
			}
		}
		return true;
	}

	/**
	 * t@CB
	 * @param file
	 */
	private void writeFile(File file) {
		BufferedWriter out = null;
		try {
			out = new BufferedWriter(new FileWriter(file));
			String text = textArea.getText();
			text = text.replaceAll("\\r\\n|\\n|\\r", lineSeparator);
			out.write(text);
			out.flush();
			statusLabel.setText(file.getAbsolutePath() + " : write ok.");
			lastPath = file.getParent();
		} catch (Exception ex) {
			ex.printStackTrace();
			statusLabel.setText(file.getAbsolutePath() + " : write error.");
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (Exception ex2) {
				}
			}
		}
	}

	/**
	 * X^CςB
	 * @param e
	 */
	public void styleFontSize(ActionEvent e) {
		JMenuItem menuItem = (JMenuItem) e.getSource();
		int size = Integer.parseInt(menuItem.getText());
		Font font = new Font("Monospaced", Font.PLAIN, size);
		textArea.setFont(font);
	}

	/**
	 * wv\B
	 *
	 */
	public void helpVersion() {
		JOptionPane.showMessageDialog(frame,
				"<html>JavaNotepad<br>Version 1.0a</html>", "JavaNotepad",
				JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * L&FXVB
	 *
	 */
	private void updateLaf() {
		laf.update(frame);
		laf.update(scrollPanel);
		laf.update(textArea);
		laf.update(statusLabel);
		laf.update(searchDialog);
		laf.update(replaceDialog);
		laf.update(popup);
	}

	/**
	 * Listener for the edits on the current document.
	 */
	private UndoableEditListener undoHandler = new UndoHandler();

	/** 
	 * UndoManager that we add edits to.
	 */
	private UndoManager undo = new UndoManager();

	private final class UndoHandler implements UndoableEditListener {

		public void undoableEditHappened(UndoableEditEvent e) {
			undo.addEdit(e.getEdit());
			undoAction.update();
			redoAction.update();
		}
	}

	/**
	 * ɖ߂B
	 * @author shiota
	 */
	private final class UndoAction extends AbstractAction {
		public UndoAction() {
			super("ɖ߂");
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			try {
				undo.undo();
			} catch (CannotUndoException ex) {
			}
			update();
			redoAction.update();
		}

		protected void update() {
			if (undo.canUndo()) {
				setEnabled(true);
				putValue(Action.NAME, undo.getUndoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, "Undo");
			}
		}
	}

	/**
	 * 蒼
	 * @author shiota
	 */
	private final class RedoAction extends AbstractAction {
		public RedoAction() {
			super("蒼");
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			try {
				undo.redo();
			} catch (CannotRedoException ex) {
			}
			update();
			undoAction.update();
		}

		protected void update() {
			if(undo.canRedo()) {
				setEnabled(true);
				putValue(Action.NAME, undo.getRedoPresentationName());
			}
			else {
				setEnabled(false);
				putValue(Action.NAME, "Redo");
			}
		}
	}

	public static void main(String[] args) {
		try {
			JavaNotepad notepad = new JavaNotepad();
			String firstFile = (args.length == 0) ? null : args[0];
			notepad.initNotepad(firstFile);
			notepad.initSearchDialog();
			notepad.initReplaceDialog();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

}
